@@ -144,7 +144,13 @@ module Data : sig
     | BLOB of string
 
   val to_string : t -> string
-  (** [to_string tp] converts column type [tp] to a string. *)
+  (** [to_string data] converts [data] to a string.  Both [NONE] and
+      [NULL] are converted to the empty string. *)
+
+  val to_string_debug : t -> string
+  (** [to_string_debug data] converts [data] to a string including the
+      data constructor.  The contents of blobs will not be printed,
+      only its length.  Useful for debugging. *)
 end
 
 
@@ -162,6 +168,12 @@ external db_close : db -> bool = "caml_sqlite3_close"
     @raise SqliteError if an invalid database handle is passed.
 *)
 
+external enable_load_extension :
+  db -> bool -> bool = "caml_sqlite3_enable_load_extension" "noalloc"
+(** [enable_load_extension db onoff] enable/disable the sqlite3 load
+    extension.  @return [false] if the operation fails, [true]
+    otherwise. *)
+
 external errcode : db -> Rc.t = "caml_sqlite3_errcode"
 (** [errcode db] @return the error code of the last operation on database
     [db].
@@ -245,6 +257,12 @@ external exec_not_null_no_headers :
     @raise SqliteError if a row contains NULL.
 *)
 
+external changes : db -> int = "caml_sqlite3_changes"
+(** [changes db] @return the number of rows that were changed
+    or inserted or deleted by the most recently completed SQL statement
+    on database [db].
+*)
+
 
 (** {2 Fine grained query operations} *)
 
@@ -310,13 +328,6 @@ external reset : stmt -> Rc.t = "caml_sqlite3_stmt_reset"
     @raise SqliteError if the statement could not be reset.
 *)
 
-external expired : stmt -> bool = "caml_sqlite3_expired"
-(** [expired stmt] @return [true] if the statement [stmt] has expired.
-    In this case it may need to be recompiled.
-
-    @raise SqliteError if the statement is invalid.
-*)
-
 
 (** {2 Data query} *)
 
@@ -327,6 +338,13 @@ external data_count : stmt -> int = "caml_sqlite3_data_count"
     @raise SqliteError if the statement is invalid.
 *)
 
+external column_count : stmt -> int = "caml_sqlite3_column_count"
+(** [column_count stmt] @return the number of columns that would be
+    returned by executing statement [stmt].
+
+    @raise SqliteError if the statement is invalid.
+*)
+
 external column : stmt -> int -> Data.t = "caml_sqlite3_column"
 (** [column stmt n] @return the data in column [n] of the
     result of the last step of statement [stmt].
@@ -336,17 +354,17 @@ external column : stmt -> int -> Data.t = "caml_sqlite3_column"
 *)
 
 external column_name : stmt -> int -> header = "caml_sqlite3_column_name"
-(** [column_name stmt n] @return the header of column [n] of the result
-    of the last step of statement [stmt].
+(** [column_name stmt n] @return the header of column [n] in the
+    result set of statement [stmt].
 
     @raise RangeError if [n] is out of range.
     @raise SqliteError if the statement is invalid.
 *)
 
 external column_decltype :
-  stmt -> int -> string = "caml_sqlite3_column_decltype"
+  stmt -> int -> string option = "caml_sqlite3_column_decltype"
 (** [column_decltype stmt n] @return the declared type of the specified
-    column of the result of the last step of statement [stmt].
+    column in the result set of statement [stmt].
 
     @raise RangeError if [n] is out of range.
     @raise SqliteError if the statement is invalid.
@@ -393,18 +411,9 @@ external bind_parameter_index :
     @raise SqliteError if the statement is invalid.
 *)
 
-external transfer_bindings :
-  stmt -> stmt -> Rc.t = "caml_sqlite3_transfer_bindings"
-(** [transfer_bindings stmt1 stmt2] transfer the bindings of statement
-    [stmt1] to [stmt2].
-
-    @return the return code of this operation.
-
-    @raise SqliteError if any of the two statements is invalid.
-*)
-
-(* TODO: does not link *)
-(* external sleep : int -> unit = "caml_sqlite3_sleep" *)
+(* TODO: these give linking errors on some platforms *)
+(* external sleep : int -> int = "caml_sqlite3_sleep" *)
+(* external clear_bindings : stmt -> Rc.t = "caml_sqlite3_clear_bindings" *)
 
 
 (** {2 Stepwise query convenience functions} *)
@@ -423,7 +432,7 @@ val row_names : stmt -> headers
     @raise SqliteError if the statement is invalid.
 *)
 
-val row_decltypes : stmt -> string array
+val row_decltypes : stmt -> string option array
 (** [row_decltypes stmt] @return all column type declarations of the
     row returned by the last query step performed with statement [stmt].
 
@@ -448,29 +457,100 @@ val create_fun0 : db -> string -> (unit -> Data.t) -> unit
 *)
 
 val create_fun1 : db -> string -> (Data.t -> Data.t) -> unit
-(** [create_funN db name f] registers function [f] under name [name]
+(** [create_fun1 db name f] registers function [f] under name [name]
     with database handle [db].  The function has arity [1].
 
     @raise SqliteError if an invalid database handle is passed.
 *)
 
 val create_fun2 : db -> string -> (Data.t -> Data.t -> Data.t) -> unit
-(** [create_funN db name f] registers function [f] under name [name]
+(** [create_fun2 db name f] registers function [f] under name [name]
     with database handle [db].  The function has arity [2].
 
     @raise SqliteError if an invalid database handle is passed.
 *)
 
 val create_fun3 : db -> string -> (Data.t -> Data.t -> Data.t-> Data.t) -> unit
-(** [create_funN db name f] registers function [f] under name [name]
+(** [create_fun3 db name f] registers function [f] under name [name]
     with database handle [db].  The function has arity [3].
 
     @raise SqliteError if an invalid database handle is passed.
 *)
 
 external delete_function : db -> string -> unit = "caml_sqlite3_delete_function"
-(** [delete_function db name] delete function with name [name] from
+(** [delete_function db name] deletes function with name [name] from
     database handle [db].
 
     @raise SqliteError if an invalid database handle is passed.
 *)
+
+val busy_timeout : db -> int -> unit
+(** [busy_timeout db ms] sets a busy handler that sleeps for a
+    specified amount of time when a table is locked.  The handler will
+    sleep multiple times until at least [ms] milliseconds of sleeping
+    have accumulated.
+
+    @raise SqliteError if an invalid database handle is passed.
+*)
+
+module Aggregate : sig
+  val create_fun0 :
+    db -> string ->
+    init : 'a ->
+    step : ('a -> 'a) ->
+    final : ('a -> Data.t) -> unit
+  (** [create_fun0 db name ~init ~step ~final] registers the step and
+      finalizer functions under name [name] with database handle [db].
+      This function has arity [0].
+
+      @raise SqliteError if an invalid database handle is passed.
+  *)
+
+  val create_fun1 :
+    db -> string ->
+    init : 'a ->
+    step : ('a -> Data.t -> 'a) ->
+    final : ('a -> Data.t) -> unit
+  (** [create_fun1 db name ~init ~step ~final] registers the step and
+      finalizer functions under name [name] with database handle [db].
+      This function has arity [1].
+
+      @raise SqliteError if an invalid database handle is passed.
+  *)
+
+  val create_fun2 :
+    db -> string ->
+    init : 'a ->
+    step : ('a -> Data.t -> Data.t -> 'a) ->
+    final : ('a -> Data.t) -> unit
+  (** [create_fun2 db name ~init ~step ~final] registers the step and
+      finalizer functions under name [name] with database handle [db].
+      This function has arity [2].
+
+      @raise SqliteError if an invalid database handle is passed.
+  *)
+
+  val create_fun3 :
+    db -> string ->
+    init : 'a ->
+    step : ('a -> Data.t -> Data.t -> Data.t -> 'a) ->
+    final : ('a -> Data.t) -> unit
+  (** [create_fun3 db name ~init ~step ~final] registers the step and
+      finalizer functions under name [name] with database handle [db].
+      This function has arity [3].
+
+      @raise SqliteError if an invalid database handle is passed.
+  *)
+
+  val create_funN :
+    db -> string ->
+    init : 'a ->
+    step : ('a -> Data.t array -> 'a) ->
+    final : ('a -> Data.t) -> unit
+  (** [create_funN db name ~init ~step ~final] registers the step and
+      finalizer functions under name [name] with database handle [db].
+      This function has arity [N].
+
+      @raise SqliteError if an invalid database handle is passed.
+  *)
+end
