@@ -318,14 +318,12 @@ CAMLprim value caml_sqlite3_open(value v_file)
 
   if (res) {
     const char *msg;
-    char buf[1024];
     if (db) {
       msg = sqlite3_errmsg(db);
       sqlite3_close(db);
     }
     else msg = "<unknown_error>";
-    snprintf(buf, sizeof(buf) - 1, "error opening database: %s", msg);
-    raise_sqlite3_Error(buf);
+    raise_sqlite3_Error("error opening database: %s", msg);
   } else if (db == NULL)
     raise_sqlite3_InternalError(
       "open returned neither a database nor an error");
@@ -363,8 +361,15 @@ CAMLprim value caml_sqlite3_enable_load_extension(value v_db, value v_onoff)
   ret = sqlite3_enable_load_extension(dbw->db, Bool_val(v_onoff));
   return Val_bool(ret);
 }
+#else
+CAMLprim value caml_sqlite3_enable_load_extension(
+  value __unused v_db, value __unused v_onoff)
+{
+  caml_failwith("enable_load_extension: unsupported");
+}
 #endif
 
+
 /* Informational functions */
 
 CAMLprim value caml_sqlite3_errcode(value v_db)
@@ -781,13 +786,6 @@ CAMLprim value caml_sqlite3_clear_bindings(value v_stmt)
 }
 #endif
 
-CAMLprim value caml_sqlite3_transfer_bindings(value v_stmt1, value v_stmt2)
-{
-  stmt_wrap *stmtw1 = safe_get_stmtw("transfer_bindings/1", v_stmt1);
-  stmt_wrap *stmtw2 = safe_get_stmtw("transfer_bindings/2", v_stmt2);
-  return Val_rc(sqlite3_transfer_bindings(stmtw1->stmt, stmtw2->stmt));
-}
-
 CAMLprim value caml_sqlite3_column_name(value v_stmt, value v_index)
 {
   CAMLparam1(v_stmt);
@@ -877,19 +875,14 @@ CAMLprim value caml_sqlite3_column(value v_stmt, value v_index)
 #if 0
 CAMLprim value caml_sqlite3_sleep(value v_duration)
 {
+  int res;
   caml_enter_blocking_section();
-    sqlite3_sleep(Int_val(v_duration));
+    res = sqlite3_sleep(Int_val(v_duration));
   caml_leave_blocking_section();
-  return Val_unit;
+  return (Int_val(res));
 }
 #endif
 
-CAMLprim value caml_sqlite3_expired(value v_stmt)
-{
-  sqlite3_stmt *stmt = safe_get_stmtw("expired", v_stmt)->stmt;
-  return sqlite3_expired(stmt) ? Val_true : Val_false;
-}
-
 
 /* User-defined functions */
 
@@ -1142,3 +1135,11 @@ CAMLprim value caml_sqlite3_busy_timeout(value v_db, value v_ms)
   if (rc != SQLITE_OK) raise_sqlite3_current(dbw->db, "busy_timeout");
   return Val_unit;
 }
+
+CAMLprim value caml_sqlite3_changes(value v_db)
+{
+  db_wrap *dbw = Sqlite3_val(v_db);
+  check_db(dbw, "changes");
+  return Val_int(sqlite3_changes(dbw->db));
+}
+
